import Buffer
import Matcher

class MalformedBoundaryError(Exception):
	pass

class NoBoundaryFoundError(Exception):
	pass

class MultipartParser(object):
	def __init__(self, fh, boundary):
		self.__fh = fh
		self.__boundary = "\n--" + boundary

		self.__buffer = Buffer.Buffer(fh)
		self.__matcher = Matcher.Matcher(self.__buffer, self.__boundary)

		self.__position = 0
		self.__boundaryOccured = False

		if True == self.__skipToBoundary():
			raise MalformedBoundaryError("Unexpected final boundary after preamble")

	def __skipToBoundary(self):
		if not self.__boundaryOccured:
			#print "Matching on " + str(self.__position) + ":" + str(self.__position + self.__buffer.BUFFER_SIZE)
			res = self.__matcher.match(self.__position + self.__buffer.BUFFER_SIZE)
			while res is None:
				read_ = len(self.__buffer.popFront(self.__buffer.BUFFER_SIZE))
				if read_ == 0:
					raise NoBoundaryFoundError()
				self.__position += read_
				#print "Matching on " + str(self.__position) + ":" + str(self.__position + self.__buffer.BUFFER_SIZE)
				res = self.__matcher.match(self.__position + self.__buffer.BUFFER_SIZE)
			#print "Match found near " + str(res)

			if res > 0 and self.__buffer[res - 1] == '\r':
				res -= 1
			self.__buffer.popFront(res - self.__position)
			self.__position = res
		return self.__skipBoundary()

	# Returns whether final boundary occured
	def __skipBoundary(self):
		finalBoundary = False

		try:
			i = self.__position
			if self.__buffer[i] == '\r':
				i += 1

			for j in xrange(0, len(self.__boundary)):
				if self.__buffer[i + j] != self.__boundary[j]:
					raise RuntimeError("skipBoundary called not on boundary position")

			i += len(self.__boundary)

			if self.__buffer[i] == '-' and self.__buffer[i+1] == '-':
				finalBoundary = True
				i += 2

			while self.__buffer[i] != '\r' and self.__buffer[i] != '\n':
				if self.__buffer[i] != ' ' and self.__buffer[i] != '\t':
					raise MalformedBoundaryError("Garbage after boundary", i)
				i += 1

			if self.__buffer[i] == '\r':
				i += 1
				if self.__buffer[i] != '\n':
					raise MalformedBoundaryError("Missing LF after CR")
			i += 1
			boundarySize = i - self.__position
			self.__matcher.skip(i)
			if len(self.__buffer.popFront(boundarySize)) != boundarySize:
				raise RuntimeError("Unable to remove boundary from buffer")
			self.__position = i
			self.__boundaryOccured = False
			return finalBoundary

		except IndexError:
			raise MalformedBoundaryError("Premature EOF")

	def read(self, size):
		if self.__boundaryOccured:
			return ""

		res = self.__matcher.match(self.__position + size + 1)
		if res == None:
			read_ = self.__buffer.popFront(size)
			if len(read_) != size:
				raise NoBoundaryFoundError("Premature EOF without closing boundary")
			self.__position += len(read_)
			return read_
		else:
			if res > 0 and self.__buffer[res - 1] == '\r':
				res -= 1
			read_ = self.__buffer.popFront(res - self.__position)
			self.__position += len(read_)
			self.__boundaryOccured = True
			return read_

	def next(self):
		return not self.__skipToBoundary()
