import glob, plesk_config, os

try:
    import subprocess
except ImportError:
    import subprocess_compat
    subprocess = subprocess_compat


class FileException(Exception):
    def __init__(self, kind, filename):
        self.kind = kind
        self.filename = filename

    def __str__(self):
        return self.filename + ": " + self.kind


class FileNotFound(FileException):
    def __init__(self, filename):
        FileException.__init__(self, "file not found", filename)


class PermissionDenied(FileException):
    def __init__(self, filename):
        FileException.__init__(self, "permission denied", filename)


class NotAFile(FileException):
    def __init__(self, filename):
        FileException.__init__(self, "not a file", filename)


def check_file(filename):
    fn = filename.encode("UTF-8")
    if not os.access(fn, os.F_OK):
        raise FileNotFound(filename)
    if not os.access(fn, os.R_OK):
        raise PermissionDenied(filename)
    if not os.path.isfile(fn):
        raise NotAFile(filename)


def findParts(base):
    """
    Raises FileNotFound if 'base' is not found, PermissionDenied if
    unable to read some parts, NotAFile if some part is a directory,
    device, pipe or a socket.
    """
    check_file(base)
    parts = [base]

    i = 1
    while True:
        filename = "%s.%03d" % (base, i)
        try:
            check_file(filename)
        except FileNotFound:
            return parts
        parts.append(filename)
        i += 1


def open(parts):
    sub = subprocess.Popen(["/bin/cat"] + [f.encode("UTF-8") for f in parts],
                           stdout = subprocess.PIPE,
                           stderr = file("/dev/null", "w"), close_fds = True)

    return sub
