import unittest
import StringIO

import Buffer

class BufferTestTemplate(object):
	def setUp(self, s):
		self.s = s
		self.f = Buffer.Buffer(StringIO.StringIO(self.s))

	def test1(self):
		self.assertEquals(self.s[:2], self.f.popFront(2))
		self.assertEquals(self.s[2], self.f.popFront(1))

	def test2(self):
		self.assertEquals(self.s[0], self.f[0])
		self.assertEquals(self.s[2], self.f[2])

	def test4(self):
		self.f.popFront(1)
		self.assertEquals(self.s[1], self.f[1])

	def test5(self):
		self.assert_(self.s == self.f.popFront(len(self.s) + 1))

	def test6(self):
		self.assertRaises(IndexError, self.f.__getitem__, len(self.s))
		self.f.popFront(1)
		self.assertRaises(IndexError, self.f.__getitem__, len(self.s))

	def test7(self):
		self.assertEquals(self.s[-1], self.f[len(self.s) - 1])
		self.f.popFront(10)
		self.assertEquals(self.s[-1], self.f[len(self.s) - 1])


class BufferTestCase(BufferTestTemplate, unittest.TestCase):
	def setUp(self):
		BufferTestTemplate.setUp(self, "abcdefghijklmnopq")


class LongBufferTestCase(BufferTestTemplate, unittest.TestCase):
	def setUp(self):
		BufferTestTemplate.setUp(self, "abcdef" + (100000 * "x") + "pq")

	def test2_1(self):
		self.f.popFront(60000)
		self.assertEquals("q", self.f[len(self.s) - 1])

	def test2_2(self):
		self.f.popFront(60000)
		self.assertRaises(IndexError, self.f.__getitem__, 0)

	def test2_3(self):
		self.f.popFront(65536)
		self.assertRaises(IndexError, self.f.__getitem__, 0)
		self.assertEquals(self.s[65536], self.f[65536])

class VeryLongBufferTestCase(BufferTestTemplate, unittest.TestCase):
	def setUp(self):
		BufferTestTemplate.setUp(self, "abcdef" + (200000 * "x") + "pq")

	def test3_1(self):
		self.f.popFront(65534)
		self.assertEquals(self.f[100000], self.s[100000])
		self.f.popFront(2)
		self.assertEquals(self.f[100000], self.s[100000])
		self.f.popFront(2)
		self.assertEquals(self.f[100000], self.s[100000])
		self.f.popFront(65534)
		self.assertEquals(self.f[200000], self.s[200000])

class DrainBuffer(unittest.TestCase):
	def test(self):
		self.s = "abcde"
		self.f = Buffer.Buffer(StringIO.StringIO(self.s))

		self.assertEquals(self.f.popFront(777), "abcde")
		for i in xrange(1, 10):
			self.assertEquals(self.f.popFront(777), "")

if __name__ == '__main__':
	unittest.main()
